import { UserSelectDialogType } from './../../../../types/userselect.dialog.type';
import {
  Component,
  OnInit,
  OnDestroy,
  Inject,
  ChangeDetectorRef,
  NgZone
} from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { NGXLogger } from 'ngx-logger';
import { Observable, Subscription, of } from 'rxjs';
import { map, tap, catchError, take } from 'rxjs/operators';

import { Store, select } from '@ngrx/store';
import * as AppStore from '@app/store';

import * as SettingsStore from '@app/store/messenger/settings';

import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { Company } from '@ucap-webmessenger/api-external';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { LoginInfo, KEY_LOGIN_INFO, KEY_VER_INFO } from '@app/types';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { UserInfo } from '@ucap-webmessenger/protocol-sync';
import {
  DeptSearchType,
  UserInfoSS,
  UserInfoF,
  UserInfoDN,
  QueryProtocolService,
  SSVC_TYPE_QUERY_DEPT_USER_DATA,
  DeptUserData,
  SSVC_TYPE_QUERY_DEPT_USER_RES
} from '@ucap-webmessenger/protocol-query';
import { UserInfo as RoomUserInfo } from '@ucap-webmessenger/protocol-room';
import {
  DialogService,
  ConfirmDialogComponent,
  ConfirmDialogResult,
  ConfirmDialogData,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap-webmessenger/ui';
import { EnvironmentsInfo, KEY_ENVIRONMENTS_INFO } from '@app/types';
import { TranslateService } from '@ngx-translate/core';
import { PiService, PromptMessageStatusCode } from '@ucap-webmessenger/pi';

export interface OrganizationContactDialogData {}

export interface OrganizationContactDialogResult {
  choice: boolean;
  selectedUser?: UserInfo | UserInfoSS | UserInfoF | UserInfoDN | RoomUserInfo;
}

@Component({
  selector: 'app-layout-messenger-organization-contact',
  templateUrl: './organization-contact.dialog.component.html',
  styleUrls: ['./organization-contact.dialog.component.scss']
})
export class OrganizationContactDialogComponent implements OnInit, OnDestroy {
  constructor(
    public dialogRef: MatDialogRef<
      OrganizationContactDialogData,
      OrganizationContactDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: OrganizationContactDialogData,
    private store: Store<any>,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private queryProtocolService: QueryProtocolService,
    private logger: NGXLogger,
    private translateService: TranslateService,
    private piService: PiService
  ) {
    this.sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
      KEY_VER_INFO
    );
    this.environmentsInfo = this.sessionStorageService.get<EnvironmentsInfo>(
      KEY_ENVIRONMENTS_INFO
    );
  }

  environmentsInfo: EnvironmentsInfo;

  UserSelectDialogType = UserSelectDialogType;

  loginRes: LoginResponse;
  loginResSubscription: Subscription;
  sessionVerinfo: VersionInfo2Response;

  // 검색
  isShowSearch = false;
  searchProcessing = false;
  searchUserInfos: UserInfoSS[] = [];

  companyList$: Observable<Company[]>;
  companyCode: string;

  ngOnInit() {
    const loginInfo = this.sessionStorageService.get<LoginInfo>(KEY_LOGIN_INFO);
    this.companyCode = loginInfo.companyCode;

    this.store.dispatch(
      SettingsStore.organizationTreeActivated({
        activate: false
      })
    );

    this.companyList$ = this.store.pipe(
      select(AppStore.SettingSelector.CompanySelector.companyList)
    );

    this.loginResSubscription = this.store
      .pipe(
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        tap(loginRes => {
          this.loginRes = loginRes;
        })
      )
      .subscribe();
  }

  ngOnDestroy(): void {
    if (!!this.loginResSubscription) {
      this.loginResSubscription.unsubscribe();
    }
  }

  /** 유저검색 */
  onKeyDownEnterOrganizationTenantSearch(params: {
    companyCode: string;
    searchWord: string;
  }) {
    if (params.searchWord.trim().length > 1) {
      this.isShowSearch = true;
      this.searchProcessing = true;

      const searchUserInfos: UserInfoSS[] = [];
      this.queryProtocolService
        .deptUser({
          divCd: 'GRP',
          companyCode: params.companyCode,
          searchRange: DeptSearchType.All,
          search: params.searchWord,
          senderCompanyCode: params.companyCode,
          senderEmployeeType: this.loginRes.userInfo.employeeType
        })
        .pipe(
          map(res => {
            switch (res.SSVC_TYPE) {
              case SSVC_TYPE_QUERY_DEPT_USER_DATA:
                const userInfos = (res as DeptUserData).userInfos;

                searchUserInfos.push(...userInfos);
                break;
              case SSVC_TYPE_QUERY_DEPT_USER_RES:
                {
                  // 검색 결과 처리.
                  this.searchUserInfos = searchUserInfos.sort((a, b) =>
                    a.name < b.name ? -1 : a.name > b.name ? 1 : 0
                  );
                  this.searchProcessing = false;
                }
                break;
            }
          }),
          catchError(error => {
            this.searchProcessing = false;
            return of(this.logger.error(error));
          })
        )
        .subscribe();
    }
  }
  /** 검색 취소 */
  onClickCancel() {
    this.isShowSearch = false;
    this.searchUserInfos = [];
  }

  onClickChoice() {
    this.dialogRef.close();
  }

  async onSendCall(callNum: string) {
    const phoneNum = callNum.replace(/\D/g, '');

    const result = await this.dialogService.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      data: {
        title: this.translateService.instant('common.messages.confirm'),
        html: this.translateService.instant('call.callWithNumber', {
          phoneNumber: callNum
        })
      }
    });

    if (!!result && !!result.choice && result.choice) {
      this.logger.debug(phoneNum);

      this.piService
        .sendCall({
          userSeq: this.loginRes.userSeq,
          deviceType: this.environmentsInfo.deviceType,
          tokenKey: this.loginRes.tokenString,
          calleeNumber: phoneNum
        })
        .pipe(
          take(1),
          map(res => {
            if (res.responseCode === PromptMessageStatusCode.Success) {
              this.logger.debug('SUCCESS');
              this.logger.debug(res);
            } else {
              this.logger.error(res);
            }
          }),
          catchError(error => of(this.logger.debug(error)))
        )
        .subscribe();
    }

    // this.dialogService.open<
    //   AlertDialogComponent,
    //   AlertDialogData,
    //   AlertDialogResult
    // >(AlertDialogComponent, {
    //   data: {
    //     title: this.translateService.instant('call.errors.label'),
    //     html: this.translateService.instant(
    //       'call.errors.cannotCallToUserWithoutPhomeNumber'
    //     )
    //   }
    // });
  }
}
