import {
  Component,
  OnInit,
  Inject,
  ViewChild,
  ElementRef,
  OnDestroy
} from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { FormGroup, FormBuilder } from '@angular/forms';
import { NGXLogger } from 'ngx-logger';
import { Store, select } from '@ngrx/store';
import * as AppStore from '@app/store';
import { Subscription, of } from 'rxjs';
import {
  DialogService,
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap-webmessenger/ui';
import {
  map,
  tap,
  catchError,
  take,
  filter,
  debounceTime
} from 'rxjs/operators';
import * as SyncStore from '@app/store/messenger/sync';
import {
  PhoneBookSyncType,
  PhoneBookSndNotification,
  PhoneBookInfo,
  KEY_PHONE_BOOK
} from '@ucap-webmessenger/protocol-sync';
import { environment } from '../../../../../environments/environment';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { TranslateService } from '@ngx-translate/core';
import {
  LocalStorageService,
  ChangedType
} from '@ucap-webmessenger/web-storage';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { PiService, PromptMessageStatusCode } from '@ucap-webmessenger/pi';
import { EnvironmentsInfo, KEY_ENVIRONMENTS_INFO } from '@app/types';

export interface PhoneBookDialogData {}

// tslint:disable-next-line: no-empty-interface
export interface PhoneBookDialogResult {
  choice: boolean;
}

@Component({
  selector: 'app-layout-messenger-phone-book',
  templateUrl: './phone-book.dialog.component.html',
  styleUrls: ['./phone-book.dialog.component.scss']
})
export class PhoneBookDialogComponent implements OnInit, OnDestroy {
  @ViewChild('messageContainer', { static: true })
  messageContainer: ElementRef;

  loginRes: LoginResponse;
  environmentsInfo: EnvironmentsInfo;

  fgSearch: FormGroup;
  tempAgeLimits = [];

  phoneBookSubscription: Subscription;
  loginResSubscription: Subscription;

  constructor(
    public dialogRef: MatDialogRef<
      PhoneBookDialogComponent,
      PhoneBookDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: PhoneBookDialogData,
    private store: Store<any>,
    private logger: NGXLogger,
    private formBuilder: FormBuilder,
    private localStorageService: LocalStorageService,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private piService: PiService
  ) {
    this.environmentsInfo = this.sessionStorageService.get<EnvironmentsInfo>(
      KEY_ENVIRONMENTS_INFO
    );
  }

  phoneBookNoti: PhoneBookSndNotification;
  isSearch: boolean;
  searchphoneBookList: PhoneBookInfo[] = [];

  ngOnInit(): void {
    this.fgSearch = this.formBuilder.group({
      searchInput: null
    });

    this.loginResSubscription = this.store
      .pipe(
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        tap(loginRes => {
          this.loginRes = loginRes;
        })
      )
      .subscribe();

    this.phoneBookSubscription = this.localStorageService.changed
      .pipe(filter(param => KEY_PHONE_BOOK === param.key))
      .subscribe(param => {
        this.logger.debug(param.key + param.type);
        if (param.type === ChangedType.Create) {
          this.onPhoneBookReload();
        } else if (param.type === ChangedType.Update) {
          if (
            !!this.phoneBookNoti &&
            this.phoneBookNoti.pageTotalCount ===
              this.phoneBookNoti.phoneBookInfos.length
          ) {
            this.onPhoneBookReload();
          }
        } else {
          this.phoneBookNoti = null;
        }
      });

    this.onPhoneBookReload();
  }

  ngOnDestroy(): void {
    if (!!this.phoneBookSubscription) {
      this.phoneBookSubscription.unsubscribe();
    }
    if (!!this.loginResSubscription) {
      this.loginResSubscription.unsubscribe();
    }
  }

  onPhoneBookReload() {
    this.phoneBookNoti = this.localStorageService.encGet<
      PhoneBookSndNotification
    >(KEY_PHONE_BOOK, environment.customConfig.appKey);
  }

  async onClickPhoneBookMenu(menuType: string) {
    switch (menuType) {
      case 'SYNC':
        {
          const result = await this.dialogService.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            width: '400px',
            data: {
              title: this.translateService.instant('call.phoneBookSync'),
              html: this.translateService.instant('call.confirmPhoneBookSync')
            }
          });

          if (!!result && !!result.choice && result.choice) {
            this.localStorageService.remove(KEY_PHONE_BOOK);

            this.store.dispatch(
              SyncStore.phoneBookReady({
                req: {
                  destination: PhoneBookSyncType.TO_MOBILE
                }
              })
            );
          }
        }
        break;
      case 'INIT':
        {
          this.localStorageService.remove(KEY_PHONE_BOOK);
        }
        break;
      default:
        break;
    }
  }

  async onClickToCall(callNumber: string) {
    const phoneNum = callNumber.replace(/\D/g, '');

    const result = await this.dialogService.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      data: {
        title: this.translateService.instant('common.messages.confirm'),
        html: this.translateService.instant('call.callWithNumber', {
          phoneNumber: callNumber
        })
      }
    });

    if (!!result && !!result.choice && result.choice) {
      this.logger.debug(phoneNum);

      this.piService
        .sendCall({
          userSeq: this.loginRes.userSeq,
          deviceType: this.environmentsInfo.deviceType,
          tokenKey: this.loginRes.tokenString,
          calleeNumber: phoneNum
        })
        .pipe(
          take(1),
          map(res => {
            if (res.responseCode === PromptMessageStatusCode.Success) {
              this.logger.debug('SUCCESS');
              this.logger.debug(res);
            } else {
              this.logger.error(res);
            }
          }),
          catchError(error => of(this.logger.debug(error)))
        )
        .subscribe();
    }
  }

  onClickChoice(): void {
    this.dialogRef.close();
  }

  onKeyDownEnter(event: KeyboardEvent, search: string) {
    event.preventDefault();
    event.stopPropagation();

    if (search.trim().length >= 1) {
      this.isSearch = true;
      this.searchphoneBookList = this.phoneBookNoti.phoneBookInfos.filter(
        phone => {
          if (phone.name.indexOf(search) >= 0) {
            return true;
          } else if (!!phone.phoneNumbers && phone.phoneNumbers.length > 0) {
            const phoneList: string[] = [];
            phone.phoneNumbers.forEach(v => {
              if (v.indexOf(search) >= 0) {
                phoneList.push(v);
              }
            });

            if (!!phoneList && phoneList.length > 0) {
              return phoneList;
            }
          } else {
            return false;
          }
        }
      );
    } else {
      this.isSearch = false;
      this.searchphoneBookList = [];
    }
  }

  onClickSearchCancel() {
    this.isSearch = false;
    this.searchphoneBookList = [];
  }
}
