import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  PacketBodyValue,
  ProtocolDecoder,
  ProtocolMessage,
  BodyStringDivider,
  ProtocolStream,
  decodeProtocolMessage,
  ProtocolNotification
} from '@ucap-webmessenger/protocol';
import { PhoneBookSyncType } from '../types/phone-book-sync.type';
import { PhoneBookInfo } from '../models/phone-book-info';
import { JsonAnalization } from '@ucap-webmessenger/api';
import { logger } from '@app/store';

export interface PhoneBookReadyRequest extends ProtocolRequest {
  // To PC, To Mobile
  destination?: PhoneBookSyncType;
}

export interface PhoneBookReadyResponse extends ProtocolResponse {
  // To PC, To Mobile
  destination?: PhoneBookSyncType;
}

export interface PhoneBookReadyNotification extends ProtocolNotification {
  // To PC, To Mobile
  destination?: PhoneBookSyncType;
}

export interface PhoneBookReadyOkRequest extends ProtocolRequest {
  // To PC, To Mobile
  syncReadyOk?: PhoneBookSyncType;
  // (W: 전체(Whole)) (W: 부분(Part))
  syncType?: PhoneBookSyncType;
}

export interface PhoneBookReadyOkResponse extends ProtocolResponse {
  // To PC, To Mobile
  syncReadyOk?: PhoneBookSyncType;
  // (W: 전체(Whole)) (W: 부분(Part))
  syncType?: PhoneBookSyncType;
}

export interface PhoneBookReadyOkNotification extends ProtocolNotification {
  // To PC, To Mobile
  syncReadyOk?: PhoneBookSyncType;
  // (W: 전체(Whole)) (W: 부분(Part))
  syncType?: PhoneBookSyncType;
}

export interface PhoneBookSndNotification extends ProtocolNotification {
  // To PC, To Mobile
  destination?: PhoneBookSyncType;
  // (W: 전체(Whole)) (W: 부분(Part))
  syncType?: PhoneBookSyncType;
  pageTotalCount?: number;
  pageListCount?: number;
  pageCurrent?: number;
  phoneBookInfos?: PhoneBookInfo[];
  returnJson: string;
}

export interface PhoneBookRcvRequest extends ProtocolRequest {
  // OK_Y, OK_N Success Type
  syncSuccessType?: PhoneBookSyncType;
  // To PC, To Mobile
  destination?: PhoneBookSyncType;
  // (W: 전체(Whole)) (W: 부분(Part))
  syncType?: PhoneBookSyncType;
  pageTotalCount?: number;
  pageListCount?: number;
  pageCurrent?: number;
}

export interface PhoneBookRcvResponse extends ProtocolResponse {
  // OK_Y, OK_N Success Type
  syncSuccessType?: PhoneBookSyncType;
  // To PC, To Mobile
  destination?: PhoneBookSyncType;
  // (W: 전체(Whole)) (W: 부분(Part))
  syncType?: PhoneBookSyncType;
  pageTotalCount?: number;
  pageListCount?: number;
  pageCurrent?: number;
}

export const encodePhoneBookReady: ProtocolEncoder<PhoneBookReadyRequest> = (
  req: PhoneBookReadyRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({
    type: PacketBodyValue.String,
    value: req.destination
  });
  return bodyList;
};

export const decodePhoneBookReady: ProtocolDecoder<PhoneBookReadyResponse> = (
  message: ProtocolMessage
) => {
  return decodeProtocolMessage(message, {
    destination: message.bodyList[0] as PhoneBookSyncType
  } as PhoneBookReadyResponse);
};

export const decodePhoneBookReadyNotification: ProtocolDecoder<PhoneBookReadyNotification> = (
  message: ProtocolMessage
) => {
  return decodeProtocolMessage(message, {
    destination: message.bodyList[0]
  } as PhoneBookReadyNotification);
};

export const encodePhoneBookReadyOk: ProtocolEncoder<PhoneBookReadyOkRequest> = (
  req: PhoneBookReadyOkRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({
    type: PacketBodyValue.String,
    value: req.syncReadyOk || ' '
  });
  bodyList.push({
    type: PacketBodyValue.String,
    value: req.syncType || ' '
  });
  return bodyList;
};

export const decodePhoneBookReadyOk: ProtocolDecoder<PhoneBookReadyResponse> = (
  message: ProtocolMessage
) => {
  return decodeProtocolMessage(message, {
    destination: message.bodyList[0],
    syncType: message.bodyList[1]
  } as PhoneBookReadyResponse);
};

export const decodePhoneBookSendNotification: ProtocolDecoder<PhoneBookSndNotification> = (
  message: ProtocolMessage
) => {
  const phoneBookInfos: PhoneBookInfo[] = [];

  const jsonBody = JsonAnalization.receiveAnalization(message.bodyList[5]);

  jsonBody.contact.forEach(info => {
    phoneBookInfos.push({
      name: info.name,
      phoneNumbers: info.phone === '' ? '' : info.phone,
      image: info.image
    });
  });

  return decodeProtocolMessage(message, {
    destination: message.bodyList[0] as PhoneBookSyncType,
    syncType: message.bodyList[1] as PhoneBookSyncType,
    pageTotalCount: message.bodyList[2],
    pageCurrent: message.bodyList[3],
    pageListCount: message.bodyList[4],
    phoneBookInfos
  } as PhoneBookSndNotification);
};

export const encodePhoneBookRcv: ProtocolEncoder<PhoneBookRcvRequest> = (
  req: PhoneBookRcvRequest
) => {
  const bodyList: PacketBody[] = [];

  bodyList.push({
    type: PacketBodyValue.String,
    value: req.syncSuccessType || ' '
  });
  bodyList.push({
    type: PacketBodyValue.String,
    value: req.destination || ' '
  });
  bodyList.push({
    type: PacketBodyValue.String,
    value: req.syncType || ' '
  });
  bodyList.push({
    type: PacketBodyValue.Integer,
    value: req.pageTotalCount || 0
  });
  bodyList.push({
    type: PacketBodyValue.Integer,
    value: req.pageCurrent || 0
  });
  bodyList.push({
    type: PacketBodyValue.Integer,
    value: req.pageListCount || 0
  });
  return bodyList;
};

export const decodePhoneBookRcv: ProtocolDecoder<PhoneBookRcvResponse> = (
  message: ProtocolMessage
) => {
  return decodeProtocolMessage(message, {
    syncSuccessType: message.bodyList[0],
    destination: message.bodyList[1],
    syncType: message.bodyList[2],
    pageTotalCount: message.bodyList[3],
    pageCurrent: message.bodyList[4],
    pageListCount: message.bodyList[5]
  } as PhoneBookRcvResponse);
};
