import { Injectable } from '@angular/core';

import { Store, select } from '@ngrx/store';
import { Observable, Subject } from 'rxjs';
import {
  map,
  takeWhile,
  filter,
  tap,
  share,
  take,
  withLatestFrom
} from 'rxjs/operators';

import { ProtocolService } from '@ucap-webmessenger/protocol';
import {
  BuddyRequest,
  BuddyResponse,
  BuddyData,
  encodeBuddy,
  decodeBuddyData,
  decodeBuddy,
  decodeBuddyDetailData,
  BuddyDetailData
} from '../protocols/buddy';
import {
  SVC_TYPE_SYNC_BUDDY,
  SSVC_TYPE_SYNC_BUDDY_REQ,
  SSVC_TYPE_SYNC_BUDDY_RES,
  SSVC_TYPE_SYNC_BUDDY_DATA,
  SSVC_TYPE_SYNC_BUDDY2_REQ,
  SSVC_TYPE_SYNC_BUDDY2_RES,
  SSVC_TYPE_SYNC_BUDDY2_DATA,
  SVC_TYPE_SYNC_GROUP,
  SSVC_TYPE_SYNC_GROUP_REQ,
  SSVC_TYPE_SYNC_GROUP_RES,
  SSVC_TYPE_SYNC_GROUP_DATA,
  SSVC_TYPE_SYNC_GROUP_REQ2,
  SSVC_TYPE_SYNC_GROUP_RES2,
  SSVC_TYPE_SYNC_GROUP_DATA2,
  SVC_TYPE_SYNC_ROOM,
  SSVC_TYPE_SYNC_ROOM_REQ,
  SSVC_TYPE_SYNC_ROOM_RES,
  SSVC_TYPE_SYNC_ROOM_DATA,
  SSVC_TYPE_SYNC_ROOM_USER,
  SSVC_TYPE_SYNC_ROOM_USER2,
  SSVC_TYPE_SYNC_ROOM2_REQ,
  SSVC_TYPE_SYNC_ROOM2_RES,
  SSVC_TYPE_SYNC_ROOM2_DATA,
  SSVC_TYPE_SYNC_ROOM2_USER,
  SSVC_TYPE_SYNC_ROOM2_USER2,
  SVC_TYPE_SYNC_CONTACT,
  SSVC_TYPE_SYNC_PHONEBOOK_READY_REQ,
  SSVC_TYPE_SYNC_PHONEBOOK_READY_RES,
  SSVC_TYPE_SYNC_PHONEBOOK_READY_NOTI,
  SSVC_TYPE_SYNC_PHONEBOOK_READY_OK_REQ,
  SSVC_TYPE_SYNC_PHONEBOOK_READY_OK_RES,
  SSVC_TYPE_SYNC_PHONEBOOK_READY_OK_NOTI,
  SSVC_TYPE_SYNC_PHONEBOOK_SND_NOTI,
  SSVC_TYPE_SYNC_PHONEBOOK_RCV_REQ,
  SSVC_TYPE_SYNC_PHONEBOOK_RCV_RES,
  SSVC_TYPE_SYNC_PHONEBOOK_RCV_NOTI
} from '../types/service';
import {
  GroupRequest,
  GroupResponse,
  GroupData,
  encodeGroup,
  decodeGroupDetailData,
  decodeGroup,
  decodeGroupData,
  GroupDetailData
} from '../protocols/group';
import {
  RoomRequest,
  RoomResponse,
  encodeRoom,
  decodeRoom,
  RoomUserData,
  RoomUserDetailData,
  RoomData,
  decodeRoomData,
  decodeRoomUserData,
  decodeRoomUserDataDetail
} from '../protocols/room';

import {
  PhoneBookReadyRequest,
  PhoneBookReadyResponse,
  PhoneBookReadyNotification,
  PhoneBookReadyOkRequest,
  PhoneBookReadyOkResponse,
  PhoneBookReadyOkNotification,
  PhoneBookSndNotification,
  PhoneBookRcvRequest,
  PhoneBookRcvResponse,
  encodePhoneBookReady,
  decodePhoneBookReady,
  decodePhoneBookReadyNotification,
  encodePhoneBookReadyOk,
  decodePhoneBookReadyOk,
  decodePhoneBookSendNotification,
  encodePhoneBookRcv,
  decodePhoneBookRcv
} from '../protocols/phone-book';
import { PhoneBookInfo } from '../models/phone-book-info';
import { NGXLogger } from 'ngx-logger';
type Notifications = PhoneBookReadyNotification | PhoneBookSndNotification;

@Injectable({
  providedIn: 'root'
})
export class SyncProtocolService {
  private notificationSubject: Subject<Notifications>;
  public notification$: Observable<Notifications>;

  constructor(
    private protocolService: ProtocolService,
    private store: Store<any>,
    private logger: NGXLogger
  ) {
    this.notificationSubject = new Subject();
    this.notification$ = this.notificationSubject.asObservable().pipe(share());

    this.protocolService.serverMessage
      .pipe(
        withLatestFrom(
          this.store.pipe(
            select((state: any) => state.messenger.sync.syncStartDate as string)
          )
        ),
        filter(
          ([message, syncStartDate]) =>
            message.serviceType === SVC_TYPE_SYNC_CONTACT
        ),
        tap(([message, syncStartDate]) => {
          switch (message.subServiceType) {
            case SSVC_TYPE_SYNC_PHONEBOOK_READY_NOTI:
              {
                this.notificationSubject.next(
                  decodePhoneBookReadyNotification(message)
                );
              }
              break;
            case SSVC_TYPE_SYNC_PHONEBOOK_SND_NOTI:
              {
                const curDate = new Date();
                const preDate = new Date(syncStartDate);

                const sec =
                  Math.abs(curDate.getTime() - preDate.getTime()) / 1000;

                this.logger.debug('SSVC_TYPE_SYNC_PHONEBOOK_SND_NOTI: ' + sec);
                this.notificationSubject.next(
                  decodePhoneBookSendNotification(message)
                );
              }
              break;
            default:
              break;
          }
        })
      )
      .subscribe();
  }
  public buddy(req: BuddyRequest): Observable<BuddyResponse | BuddyData> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_BUDDY, SSVC_TYPE_SYNC_BUDDY_REQ, ...encodeBuddy(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_BUDDY_RES !== res.subServiceType, true),
        map(res => {
          if (SSVC_TYPE_SYNC_BUDDY_DATA === res.subServiceType) {
            return decodeBuddyData(res);
          }

          return decodeBuddy(res);
        })
      );
  }

  public buddy2(
    req: BuddyRequest
  ): Observable<BuddyResponse | BuddyDetailData> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_BUDDY, SSVC_TYPE_SYNC_BUDDY2_REQ, ...encodeBuddy(req))
      .pipe(
        takeWhile(
          res => SSVC_TYPE_SYNC_BUDDY2_RES !== res.subServiceType,
          true
        ),
        map(res => {
          if (SSVC_TYPE_SYNC_BUDDY2_DATA === res.subServiceType) {
            return decodeBuddyDetailData(res);
          }

          return decodeBuddy(res);
        })
      );
  }

  public group(req: GroupRequest): Observable<GroupResponse | GroupData> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_GROUP, SSVC_TYPE_SYNC_GROUP_REQ, ...encodeGroup(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_GROUP_RES !== res.subServiceType, true),
        map(res => {
          if (SSVC_TYPE_SYNC_GROUP_DATA === res.subServiceType) {
            return decodeGroupData(res);
          }

          return decodeGroup(res);
        })
      );
  }

  public group2(
    req: GroupRequest
  ): Observable<GroupResponse | GroupDetailData> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_GROUP, SSVC_TYPE_SYNC_GROUP_REQ2, ...encodeGroup(req))
      .pipe(
        takeWhile(
          res => SSVC_TYPE_SYNC_GROUP_RES2 !== res.subServiceType,
          true
        ),
        map(res => {
          if (SSVC_TYPE_SYNC_GROUP_DATA2 === res.subServiceType) {
            return decodeGroupDetailData(res);
          }

          return decodeGroup(res);
        })
      );
  }

  public room(
    req: RoomRequest
  ): Observable<RoomResponse | RoomData | RoomUserData | RoomUserDetailData> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_ROOM, SSVC_TYPE_SYNC_ROOM_REQ, ...encodeRoom(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_ROOM_RES !== res.subServiceType, true),
        map(res => {
          if (SSVC_TYPE_SYNC_ROOM_DATA === res.subServiceType) {
            return decodeRoomData(res);
          } else if (SSVC_TYPE_SYNC_ROOM_USER === res.subServiceType) {
            return decodeRoomUserData(res);
          } else if (SSVC_TYPE_SYNC_ROOM_USER2 === res.subServiceType) {
            return decodeRoomUserDataDetail(res);
          }

          return decodeRoom(res);
        })
      );
  }

  public room2(
    req: RoomRequest
  ): Observable<RoomResponse | RoomData | RoomUserData | RoomUserDetailData> {
    return this.protocolService
      .call(SVC_TYPE_SYNC_ROOM, SSVC_TYPE_SYNC_ROOM2_REQ, ...encodeRoom(req))
      .pipe(
        takeWhile(res => SSVC_TYPE_SYNC_ROOM2_RES !== res.subServiceType, true),
        map(res => {
          if (SSVC_TYPE_SYNC_ROOM2_DATA === res.subServiceType) {
            return decodeRoomData(res);
          } else if (SSVC_TYPE_SYNC_ROOM2_USER === res.subServiceType) {
            return decodeRoomUserData(res);
          } else if (SSVC_TYPE_SYNC_ROOM2_USER2 === res.subServiceType) {
            return decodeRoomUserDataDetail(res);
          }

          return decodeRoom(res);
        })
      );
  }

  public phoneBookReady(
    req: PhoneBookReadyRequest
  ): Observable<PhoneBookReadyResponse> {
    return this.protocolService
      .call(
        SVC_TYPE_SYNC_CONTACT,
        SSVC_TYPE_SYNC_PHONEBOOK_READY_REQ,
        ...encodePhoneBookReady(req)
      )
      .pipe(
        take(1),
        map(res => {
          return decodePhoneBookReady(res);
        })
      );
  }

  public phoneBookReadyOk(
    req: PhoneBookReadyOkRequest
  ): Observable<PhoneBookReadyOkResponse> {
    return this.protocolService
      .call(
        SVC_TYPE_SYNC_CONTACT,
        SSVC_TYPE_SYNC_PHONEBOOK_READY_OK_REQ,
        ...encodePhoneBookReadyOk(req)
      )
      .pipe(
        takeWhile(
          res => SSVC_TYPE_SYNC_PHONEBOOK_READY_OK_RES !== res.subServiceType,
          true
        ),
        map(res => {
          return decodePhoneBookReadyOk(res);
        })
      );
  }

  public phoneBookRcv(
    req: PhoneBookRcvRequest
  ): Observable<PhoneBookRcvResponse> {
    return this.protocolService
      .call(
        SVC_TYPE_SYNC_CONTACT,
        SSVC_TYPE_SYNC_PHONEBOOK_RCV_REQ,
        ...encodePhoneBookRcv(req)
      )
      .pipe(
        take(1),
        map(res => {
          return decodePhoneBookRcv(res);
        })
      );
  }
}
