import {
  Component,
  OnInit,
  EventEmitter,
  Output,
  ViewChild,
  ElementRef
} from '@angular/core';
import { NGXLogger } from 'ngx-logger';
import { Store, select } from '@ngrx/store';
import * as AppStore from '@app/store';
import {
  Observable,
  combineLatest,
  Subscription,
  of,
  BehaviorSubject
} from 'rxjs';
import { map, tap, catchError, take } from 'rxjs/operators';
import { asYouType } from 'libphonenumber-js';
import { PiService, PromptMessageStatusCode } from '@ucap-webmessenger/pi';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { EnvironmentsInfo, KEY_ENVIRONMENTS_INFO } from '@app/types';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import {
  DialogService,
  ConfirmDialogComponent,
  ConfirmDialogResult,
  ConfirmDialogData,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap-webmessenger/ui';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'ucap-call-dialpad',
  templateUrl: './dialpad.component.html',
  styleUrls: ['./dialpad.component.scss']
})
export class DialpadComponent implements OnInit {
  @Output()
  sendCall = new EventEmitter<string>();

  // @Output()
  // close = new EventEmitter();

  @ViewChild('phoneInput', { static: false })
  phoneInput: ElementRef;

  printNumber: string;
  inputNumber: string;
  isShowRemoveBtn: boolean;
  isShowTempBtn: boolean;

  environmentsInfo: EnvironmentsInfo;

  loginRes: LoginResponse;
  loginResSubscription: Subscription;

  constructor(
    private logger: NGXLogger,
    private store: Store<any>,
    private piService: PiService,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private translateService: TranslateService
  ) {
    this.environmentsInfo = this.sessionStorageService.get<EnvironmentsInfo>(
      KEY_ENVIRONMENTS_INFO
    );
  }

  onBlur(event) {
    // this.phoneInput.nativeElement.focus();
  }

  ngOnInit() {
    this.isShowRemoveBtn = false;
    this.isShowTempBtn = false;
    this.inputNumber = '';
    this.printNumber = '';

    this.loginResSubscription = this.store
      .pipe(
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        tap(loginRes => {
          this.loginRes = loginRes;
        })
      )
      .subscribe();
  }

  onClickNumber(num: string) {
    if (!this.isShowRemoveBtn) {
      this.isShowRemoveBtn = true;
    }

    this.inputNumber = this.inputNumber.concat(num);

    if (this.getSpecialSymbol(this.inputNumber)) {
      this.printNumber = this.inputNumber;
    } else {
      this.printNumber = new asYouType('KR').input(this.inputNumber);
    }
  }

  onClickRemove() {
    this.inputNumber = this.inputNumber.substring(
      0,
      this.inputNumber.length - 1
    );

    if (this.inputNumber.length === 0) {
      this.isShowRemoveBtn = false;
    }

    const isSymbol = this.getSpecialSymbol(
      this.inputNumber.substring(this.inputNumber.length - 1)
    );
    if (isSymbol) {
      this.printNumber = this.inputNumber;
    } else {
      this.printNumber = new asYouType('KR').input(this.inputNumber);
    }
  }
  async onClickCall() {
    const phoneNum = this.printNumber.replace(/\D/g, '');

    const result = await this.dialogService.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      data: {
        title: this.translateService.instant('common.messages.confirm'),
        html: this.translateService.instant('call.callWithNumber', {
          phoneNumber: phoneNum
        })
      }
    });

    if (!!result && !!result.choice && result.choice) {
      this.piService
        .sendCall({
          userSeq: this.loginRes.userSeq,
          deviceType: this.environmentsInfo.deviceType,
          tokenKey: this.loginRes.tokenString,
          calleeNumber: phoneNum
        })
        .pipe(
          take(1),
          map(res => {
            if (res.responseCode === PromptMessageStatusCode.Success) {
              this.logger.debug('SUCCESS');
              this.logger.debug(res);
            } else {
              this.logger.error(res);
            }
          }),
          catchError(error => of(this.logger.debug(error)))
        )
        .subscribe();
    }
  }

  onClickAllRemove() {
    this.inputNumber = '';
    this.printNumber = '';
    this.isShowRemoveBtn = false;
  }

  onChange(e: KeyboardEvent) {
    if (e.keyCode > 47 && e.keyCode < 58) {
      this.onClickNumber(e.key);
    } else if (e.keyCode === 8) {
      this.onClickRemove();
    } else if (
      0 === e.key.localeCompare('*') ||
      0 === e.key.localeCompare('#')
    ) {
      this.onClickNumber(e.key);
    }
    return;
  }

  private getSpecialSymbol(num: string): boolean {
    const asteriskIndex = this.inputNumber.indexOf('*');
    const hashIndex = this.inputNumber.indexOf('#');

    if (
      asteriskIndex > -1 ||
      hashIndex > -1 ||
      0 === num.localeCompare('*') ||
      0 === num.localeCompare('#')
    ) {
      return true;
    } else {
      return false;
    }
  }
}
