import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { ucapAnimations } from '../animations';
import {
  FileEventJson,
  BundleImageEventJson
} from '@ucap-webmessenger/protocol-event';
import { FileViewerType } from '../types/file-viewer.type';
import { FileType, FileInfo, isSound } from '@ucap-webmessenger/protocol-file';
import { FileDownloadItem } from '@ucap-webmessenger/api';
import { SelectFileInfo } from '../models/select-file-info';

@Component({
  selector: 'ucap-media-viewer',
  templateUrl: './media-viewer.component.html',
  styleUrls: ['./media-viewer.component.scss'],
  animations: ucapAnimations
})
export class MediaViewerComponent implements OnInit {
  @Input()
  set fileInfo(v: { fileInfos: FileInfo[]; selectFileInfo: SelectFileInfo }) {
    this.fileInfos = v.fileInfos.sort((a, b) => a.eventSeq - b.eventSeq);

    this.fileInfoIndex = this.fileInfos.findIndex(
      f => f.sentMessageJson.attachmentSeq === v.selectFileInfo.attachmentSeq
    );
    this.bundleIndex = v.selectFileInfo.index;

    this.setCuttentFileInfo();
  }

  @Input()
  fileDownloadUrl: (attachmentSeq: number) => string;

  @Output()
  download = new EventEmitter<{
    attachmentSeq?: number;
    downloadUrl?: string;
    fileName: string;
    fileDownloadItem: FileDownloadItem;
  }>();

  @Output()
  closed = new EventEmitter<void>();

  fileInfos: FileInfo[];
  currentFileInfo: FileInfo;
  fileInfoIndex: number;
  bundleIndex: number;
  FileType = FileType;
  FileViewerType = FileViewerType;

  get downloadUrl() {
    if (FileType.Bundle === this.currentFileInfo.type) {
      const fileJsonInfo = this.currentFileInfo
        .sentMessageJson as BundleImageEventJson;
      return `${fileJsonInfo.baseUrl}${
        fileJsonInfo.thumbUrls[this.bundleIndex]
      }`
        .replace('WebFile', 'AttFile')
        .replace('.thumb.jpg', '');
    } else {
      return this.fileDownloadUrl(this.currentFileInfo.seq);
    }
  }

  get fileName() {
    if (FileType.Bundle === this.currentFileInfo.type) {
      const fileJsonInfo = this.currentFileInfo
        .sentMessageJson as BundleImageEventJson;
      return `${fileJsonInfo.thumbUrls[this.bundleIndex]}`.replace(
        '.thumb.jpg',
        ''
      );
    } else {
      return this.currentFileInfo.sentMessageJson.fileName;
    }
  }

  constructor() {}

  ngOnInit() {}

  detectFileViewerType(fileInfo: FileInfo): FileViewerType {
    switch (fileInfo.type) {
      case FileType.Image:
      case FileType.Bundle:
        return FileViewerType.Image;
      case FileType.Sound:
        return FileViewerType.Sound;
      case FileType.Video:
        return FileViewerType.Video;
      default:
        if (isSound(fileInfo)) {
          return FileViewerType.Sound;
        }
        return FileViewerType.Binary;
    }
  }

  onDownload(fileDownloadItem: FileDownloadItem): void {
    this.download.emit(
      FileType.Bundle !== this.currentFileInfo.type
        ? {
            attachmentSeq: this.currentFileInfo.sentMessageJson.attachmentSeq,
            fileName: this.currentFileInfo.sentMessageJson.fileName,
            fileDownloadItem
          }
        : {
            downloadUrl: this.downloadUrl,
            fileName: this.fileName,
            fileDownloadItem
          }
    );
  }

  onClosedViewer(): void {
    this.closed.emit();
  }

  onClickPrev() {
    if (
      0 === this.fileInfoIndex &&
      (FileType.Bundle !== this.currentFileInfo.type ||
        (FileType.Bundle === this.currentFileInfo.type &&
          0 === this.bundleIndex))
    ) {
      return;
    }

    if (FileType.Bundle === this.currentFileInfo.type) {
      if (0 === this.bundleIndex) {
        this.fileInfoIndex--;
        this.bundleIndex = undefined;
      } else {
        this.bundleIndex--;
      }
    } else {
      this.fileInfoIndex--;
      this.bundleIndex = undefined;
    }

    this.setCuttentFileInfo(false);
  }

  onClickNext() {
    if (
      this.fileInfos.length - 1 === this.fileInfoIndex &&
      (FileType.Bundle !== this.currentFileInfo.type ||
        (FileType.Bundle === this.currentFileInfo.type &&
          this.bundleImage.fileCount - 1 === this.bundleIndex))
    ) {
      return;
    }

    if (FileType.Bundle === this.currentFileInfo.type) {
      const fileJsonInfo = this.currentFileInfo
        .sentMessageJson as BundleImageEventJson;
      if (fileJsonInfo.fileCount - 1 === this.bundleIndex) {
        this.fileInfoIndex++;
        this.bundleIndex = undefined;
      } else {
        this.bundleIndex++;
      }
    } else {
      this.fileInfoIndex++;
      this.bundleIndex = undefined;
    }

    this.setCuttentFileInfo(true);
  }

  get bundleImage(): BundleImageEventJson {
    return this.currentFileInfo.sentMessageJson as BundleImageEventJson;
  }

  private setCuttentFileInfo(direction?: boolean) {
    this.currentFileInfo = this.fileInfos[this.fileInfoIndex];
    if (
      FileType.Bundle === this.currentFileInfo.type &&
      undefined === this.bundleIndex
    ) {
      if (undefined === direction) {
        return;
      }
      const fileJsonInfo = this.currentFileInfo
        .sentMessageJson as BundleImageEventJson;
      if (direction) {
        // Next
        this.bundleIndex = 0;
      } else {
        // Prev
        this.bundleIndex = fileJsonInfo.fileCount - 1;
      }
    }
  }
}
