import { StorageUtil, ExpiredUnit } from '../utils/storage.util';
import { Observable, Subject } from 'rxjs';
import { share } from 'rxjs/operators';

import CryptoJS from 'crypto-js';
import { NGXLogger } from 'ngx-logger';

export enum ChangedType {
  Create = 'Create',
  Update = 'Update',
  Delete = 'Delete'
}

export class StorageService {
  private changedSubject: Subject<{
    key: string;
    type: ChangedType;
  }> | null = null;
  private changed$: Observable<{
    key: string;
    type: ChangedType;
  }> | null = null;

  constructor(private storage: Storage, private logger: NGXLogger) {
    this.changedSubject = new Subject();
    this.changed$ = this.changedSubject.asObservable().pipe(share());
  }

  get<T>(key: string): T | null {
    return StorageUtil.get(this.storage, key);
  }

  set<T>(
    key: string,
    value: T,
    expiredAt: number = 0,
    expiredUnit: ExpiredUnit = 'd'
  ) {
    const old = StorageUtil.get(this.storage, key);

    StorageUtil.set(this.storage, key, value, expiredAt, expiredUnit);

    this.changedSubject.next({
      key,
      type: null === old ? ChangedType.Create : ChangedType.Update
    });
  }

  encGet<T>(key: string, secretPassphrase: string): T | null {
    const encrypted = StorageUtil.get(this.storage, key) as CryptoJS.WordArray;
    if (!encrypted) {
      return null;
    }
    const decrypted = CryptoJS.AES.decrypt(encrypted, secretPassphrase);
    const json = decrypted.toString(CryptoJS.enc.Utf8);

    return JSON.parse(json) as T;
  }

  encSet<T>(
    key: string,
    value: T,
    secretPassphrase: string,
    expiredAt: number = 0,
    expiredUnit: ExpiredUnit = 'd'
  ) {
    const old = StorageUtil.get(this.storage, key);

    this.logger.debug('encSet key:', key, ' value:', value);
    const json = JSON.stringify(value);
    const encrypted = CryptoJS.AES.encrypt(json, secretPassphrase);
    StorageUtil.set(
      this.storage,
      key,
      encrypted.toString(),
      expiredAt,
      expiredUnit
    );

    this.changedSubject.next({
      key,
      type: null === old ? ChangedType.Create : ChangedType.Update
    });
  }

  remove(key: string) {
    let index = 0;
    let next = StorageUtil.key(this.storage, index);
    let exist = false;
    while (next) {
      if (next === key) {
        exist = true;
        break;
      }
      next = StorageUtil.key(this.storage, ++index);
    }

    if (exist) {
      StorageUtil.remove(this.storage, key);
      this.changedSubject.next({ key, type: ChangedType.Delete });
    }
  }

  clear() {
    let index = 0;
    let next = StorageUtil.key(this.storage, index);
    const ls: string[] = [];
    while (next) {
      ls.push(next);
      next = StorageUtil.key(this.storage, ++index);
    }
    this.storage.clear();

    ls.forEach(key => {
      this.changedSubject.next({ key, type: ChangedType.Delete });
    });
  }

  // observe(key: string): Observable<StorageEvent> {
  //   return fromEvent<StorageEvent>(window, 'storage').pipe(
  //     filter(event => event.storageArea === this.storage),
  //     filter(event => event.key === key)
  //   );
  // }

  /**
   * localStorageService.changed
   *  .pipe(filter(param => 'EXAMPLE' === param.key))
   *  .subscribe(param => {
   *    console.log('changed');
   *  });
   */
  public get changed(): Observable<{
    key: string;
    type: ChangedType;
  }> {
    return this.changed$;
  }
}
